
#include "TestRenderThread.h"

#if defined QT_OPENGL_ES_1

#include <QtOpenGL>
#include <GLES/glext.h>

#elif defined QT_OPENGL_ES_2

#include <QtOpenGL>
#include <GLES2/gl2ext.h>

#endif

// Qt Headers
#pragma warning(push, 1)
#include <QtGlobal>
#include <QtOpenGL>
#include <QOpenGLContext>
#include <QOpenGLShaderProgram>
#include <QThread>
#include <QEvent>
#pragma warning(pop)



/***************************************************************************/

tTestRenderThread::tTestRenderThread(QOpenGLWindow* pRenderTarget):
	m_pRenderTarget(pRenderTarget),
	m_pContext(0),
	m_Initialize(true)
{
	Q_ASSERT(m_pRenderTarget != 0);

	if (m_pRenderTarget == 0)
	{
		Q_ASSERT("Cannot create a render thread without a valid rendering target!");
	}
}

/***************************************************************************/

tTestRenderThread::~tTestRenderThread()
{
    m_pContext->doneCurrent();
    delete m_pContext;
    m_pContext = 0;

	quit();
	wait();
}

/***************************************************************************/

void tTestRenderThread::run()
{
	qDebug("Entering run()\n");

    InitializeContext();
    Initialize();  // Initialize the render system in the current thread.

    // Make the context current for worker thread.
    m_pContext->makeCurrent(m_pRenderTarget);

	// Test simple OpenGL ES command
    glClearColor(0.0f, 0.1f, 0.0f, 1.0f);

    // Start the thread's event loop
    exec();

    delete m_pShaderProgram;
}

/***************************************************************************/

void tTestRenderThread::customEvent(QEvent* pEvent)
{
	// For testing, there is only one event -- render.

	qDebug("Custom Event received in Rendering Thread.\n");
    pEvent->accept();

	RenderScene();
}

/***************************************************************************/

void tTestRenderThread::RenderScene()
{
	qDebug("Entering RenderScene()\n");

    m_pContext->makeCurrent(m_pRenderTarget);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glViewport(0, 0, m_pRenderTarget->width(), m_pRenderTarget->height());

#if (defined Q_WS_QPA && !defined QT_OPENGL_ES_1) || defined QT_OPENGL_ES_2 || QTOPENGL_VERSION >= 2 || defined Q_OS_WIN32

    static QVector2D vertices[4];
    vertices[0] = QVector2D(0, 1);
    vertices[1] = QVector2D(0, 0);
    vertices[2] = QVector2D(1, 1);
    vertices[3] = QVector2D(1, 0);

    static QVector4D colors[4];
    colors[0] = QVector4D(1, 0, 0, 1);
    colors[1] = QVector4D(0, 0, 0, 1);
    colors[2] = QVector4D(0, 0, 0, 1);
    colors[3] = QVector4D(0, 0, 1, 1);

    QMatrix4x4 projection;
    projection.ortho(0, m_pRenderTarget->width(), 0, m_pRenderTarget->height(), -1, 1);
    projection.scale(m_pRenderTarget->width(), m_pRenderTarget->height(), 0);

    m_pShaderProgram->bind();

    m_pShaderProgram->setUniformValue(matrixUL, projection);

    m_pShaderProgram->enableAttributeArray(vertexAL);
    m_pShaderProgram->enableAttributeArray(colorAL);
    m_pShaderProgram->setAttributeArray(vertexAL, vertices);
    m_pShaderProgram->setAttributeArray(colorAL, colors);

#else  // Begin legacy rendering

#if !defined(Q_WS_QWS) && !defined(QT_OPENGL_ES_1)
	glOrtho(0, m_pRenderTarget->width(), 0, m_pRenderTarget->height(), -1, 1);
#else
    glOrthof(0, m_pRenderTarget->width(), 0, m_pRenderTarget->height(), -1, 1);
#endif

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

	// Draw Test Quad
	static float vertices[] = { 0,1, 0,0, 1,1, 1,0 };
    static float colors[] = { 1,0,0,1, 0,0,0,1, 0,0,0,1, 0,0,1,1 };

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_COLOR_ARRAY);

    glVertexPointer(2, GL_FLOAT, 0, vertices);
    glColorPointer(4, GL_FLOAT, 0, colors);

	glScalef(m_pRenderTarget->width(), m_pRenderTarget->height(), 1);

#endif

	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    m_pContext->swapBuffers(m_pRenderTarget);

	qDebug("RenderScene() finished.");
}

/***************************************************************************/

void tTestRenderThread::InitializeContext()
{
    m_pContext = new QOpenGLContext();
    m_pContext->create();

    if (m_pContext->isValid() == false)
    {
        Q_ASSERT("Unable to create OpenGL context!  Render thread creation failed.");
    }

    m_pContext->doneCurrent();

	qDebug("Initializing OpenGL ES Context -- makeCurrent()\n");
}

/***************************************************************************/

void tTestRenderThread::Initialize()
{
    m_pContext->makeCurrent(m_pRenderTarget);

    glClearColor(1.0f, 0.1f, 0.0f, 1.0f);
    glEnable(GL_DEPTH_TEST);

#if (defined Q_WS_QPA && !defined QT_OPENGL_ES_1) || defined QT_OPENGL_ES_2 || QTOPENGL_VERSION >= 2 || defined Q_OS_WIN32
    
    m_pShaderProgram = new QOpenGLShaderProgram(this);
    m_pShaderProgram->addShaderFromSourceFile(QOpenGLShader::Vertex, ":/shaders/VertexShader.glsl");
    m_pShaderProgram->addShaderFromSourceFile(QOpenGLShader::Fragment, ":/shaders/FragmentShader.glsl");
    m_pShaderProgram->link();

    vertexAL = m_pShaderProgram->attributeLocation("vertex");
    colorAL = m_pShaderProgram->attributeLocation("inColor");
    matrixUL = m_pShaderProgram->uniformLocation("matrix");

#endif
}





